%%
%% MATLAB TEST FILE FOR ACTIVEX COMPONENT 
%%              NORMALFLOW
%%
%% TESTING MODE : CHANGEDETECTION
%%

disp('CREATE IMAGE SEQUENCE');
i1 = uint8(zeros(240,320));
itemp = uint8(zeros(240,320));
itemp(130:140,140:160) = 255;
i2 = imrotate(itemp, 260, 'bicubic','crop');

disp('GET IMAGE DIMENSION');		
imgsize = size(i1);					
   
disp('CREATE NORMALFLOW ACTIVEX SERVER OBJECT');
h = actxserver('NormalFlow.CoNormalFlow');

disp('CONFIGURE OBJECT COMPATIBLE WITH IMAGE SIZE');
h.lines = imgsize(1);		
h.columns = imgsize(2);		
h.computation_mode = 'changedetection';
h.timederivative_mode = 'recursive';

disp('ALLOCATE ACTIVEX INTERNAL RESOURCES');
invoke(h,'create_normal_flow');

%Feed first image on the filter
disp('NOTICE THAT MATLAB REPRESENTS DATA COLUMNWISE. USE THE TRANSPOSE');	
invoke(h,'process_data',i1');
invoke(h,'process_data',i1');
invoke(h,'process_data',i1');
invoke(h,'process_data',i1');

%Feed second image on the filter
invoke(h,'process_data',i2');
stats = invoke(h,'get_energy_statistics');
A = stats(1);
X = stats(2);
Y = stats(3);
momentX = stats(4);
momentY = stats(5);
momentXY = stats(6);

disp('DISPLAY COMPUTED IMAGES');
figure; imshow(h.spacesmooth',[]);set(gcf,'Name','spacesmooth');
figure; imshow(h.timesmooth',[]);set(gcf,'Name','timesmooth');
figure; imshow(h.horzderiv',[]);set(gcf,'Name','horzderiv');
figure; imshow(h.vertderiv',[]);set(gcf,'Name','vertderiv');
figure; imshow(h.timederiv',[]);set(gcf,'Name','timederiv');
figure; imshow(h.squaredtimederiv',[]);set(gcf,'Name','squaredtimederiv');

figure; imshow(i2,[]);set(gcf,'Name','Motion Energy Moments');

hold on; 
if A ~= 0,
	%covariance ellipse estimation
	SigmaX2= momentX/A;
	SigmaX = sqrt(SigmaX2);
	SigmaY2 = momentY/A;
	SigmaY = sqrt(SigmaX2);
	SigmaXY = momentXY/A;
	Corr = SigmaXY/SigmaX/SigmaY;
	Corr2 = Corr.^2;
	%axes orientation
	Angle = 0.5*atan(2*Corr*SigmaX*SigmaY/(SigmaX2-SigmaY2));
	%principal semi-diameters
	Big2 = SigmaX2*SigmaY2*(1-Corr2)/(SigmaY2*cos(Angle)^2-2*Corr*SigmaX*SigmaY*sin(Angle)*cos(Angle)+SigmaX2*sin(Angle)^2);
	Big = sqrt(Big2);
	Small2 = SigmaX2*SigmaY2*(1-Corr2)/(SigmaY2*sin(Angle)^2+2*Corr*SigmaX*SigmaY*sin(Angle)*cos(Angle)+SigmaX2*cos(Angle)^2);
	Small = sqrt(Small2);

	BigXmin = X - Big*cos(Angle);
	BigXmax = X + Big*cos(Angle);
	BigYmin = Y - Big*sin(Angle);
	BigYmax = Y + Big*sin(Angle);

	SmallXmin = X - Small*sin(Angle);
	SmallXmax = X + Small*sin(Angle);
	SmallYmin = Y + Small*cos(Angle);
	SmallYmax = Y - Small*cos(Angle);

	plot([BigXmin BigXmax]+1, [BigYmin BigYmax]+1,'r-');
	plot([SmallXmin SmallXmax]+1, [SmallYmin SmallYmax]+1,'r-');

end;
hold off; 

disp('CLOSE OBJECT');
invoke(h,'destroy_normal_flow');
delete(h);

%%
%% Copyright (c) 2002, Alexandre Bernardino - ISR/IST.  All rights reserved.
%% alex@isr.ist.utl.pt
%%

