%%
%% MATLAB TEST FILE FOR ACTIVEX COMPONENT 
%%              NORMALFLOW
%%
%% TESTING MODE : FLOWVALIDATION
%%

disp('CREATE IMAGE SEQUENCE');
[x,y] = meshgrid(1:320,1:240);
i1 = uint8((sin(x/20)+cos(y/20)+2)*64);
i2 = uint8((sin(x/20+0.25)+cos(y/20+0.25)+2)*64);

disp('GET IMAGE DIMENSION');		
imgsize = size(i1);					
   
disp('CREATE NORMALFLOW ACTIVEX SERVER OBJECT');
h = actxserver('NormalFlow.CoNormalFlow');

disp('CONFIGURE OBJECT COMPATIBLE WITH IMAGE SIZE');
h.lines = imgsize(1);		
h.columns = imgsize(2);		
h.computation_mode = 'flowvalidation';
h.timederivative_mode = 'recursive';
h.minvalidgrad = 3;
h.minvalidtimederiv = 1;
h.minvalidflowmag = 0.01;

disp('ALLOCATE ACTIVEX INTERNAL RESOURCES');
invoke(h,'create_normal_flow');

%Feed first image on the filter
disp('NOTICE THAT MATLAB REPRESENTS DATA COLUMNWISE. USE THE TRANSPOSE');	
invoke(h,'process_data',i1');

%Feed second image on the filter
invoke(h,'process_data',i2');

disp('DISPLAY COMPUTED IMAGES');
figure; imshow(h.spacesmooth',[]);set(gcf,'Name','spacesmooth');
figure; imshow(h.timesmooth',[]);set(gcf,'Name','timesmooth');
figure; imshow(h.horzderiv',[]);set(gcf,'Name','horzderiv');
figure; imshow(h.vertderiv',[]);set(gcf,'Name','vertderiv');
figure; imshow(h.timederiv',[]);set(gcf,'Name','timederiv');
figure; imshow(h.squaredtimederiv',[]);set(gcf,'Name','squaredtimederiv');
figure; imshow(h.squaredhorzderiv',[]);set(gcf,'Name','squaredhorzderiv');
figure; imshow(h.squaredvertderiv',[]);set(gcf,'Name','squaredvertderiv');
figure; imshow(h.horztimederiv',[]);set(gcf,'Name','horztimederiv');
figure; imshow(h.verttimederiv',[]);set(gcf,'Name','verttimederiv');
figure; imshow(h.horzvertderiv',[]);set(gcf,'Name','horzvertderiv');
figure; imshow(h.squaredgradient',[]);set(gcf,'Name','squaredgradient');
figure; imshow(h.validdatamask',[]);set(gcf,'Name','validdatamask');

disp('CLOSE OBJECT');
invoke(h,'destroy_normal_flow');
delete(h);

%%
%% Copyright (c) 2002, Alexandre Bernardino - ISR/IST.  All rights reserved.
%% alex@isr.ist.utl.pt
%%

