%%
%% MATLAB TEST FILE FOR ACTIVEX COMPONENT 
%%              NORMALFLOW
%%
%% TESTING MODE : NORMALFLOW
%%

disp('CREATE IMAGE SEQUENCE');
[x,y] = meshgrid(1:320,1:240);
i1 = uint8((sin(x/20)+cos(y/20)+2)*64);
i2 = uint8((sin(x/20+0.25)+cos(y/20+0.25)+2)*64);

disp('GET IMAGE DIMENSION');		
imgsize = size(i1);					
   
disp('CREATE NORMALFLOW ACTIVEX SERVER OBJECT');
h = actxserver('NormalFlow.CoNormalFlow');

disp('CONFIGURE OBJECT COMPATIBLE WITH IMAGE SIZE');
h.lines = imgsize(1);		
h.columns = imgsize(2);		
h.computation_mode = 'normalflow';
h.timederivative_mode = 'recursive';
h.minvalidgrad = 3;
h.minvalidtimederiv = 1;
h.minvalidflowmag = 0.01;

disp('ALLOCATE ACTIVEX INTERNAL RESOURCES');
invoke(h,'create_normal_flow');

%Feed first image on the filter
disp('NOTICE THAT MATLAB REPRESENTS DATA COLUMNWISE. USE THE TRANSPOSE');	
invoke(h,'process_data',i1');

%Feed second image on the filter
invoke(h,'process_data',i2');

u = double(h.horzflow);
v = double(h.vertflow);

%show images
figure; imshow(u',[]);set(gcf,'Name','horzflow');
figure; imshow(v',[]);set(gcf,'Name','vertflow');


%[x,y] = meshgrid(1:double(h.lines),1:double(h.columns));
figure;
imshow(h.timesmooth',[]);set(gcf,'Name','Normal Flow');
hold on;
quiver(x(1:10:end,1:10:end)',y(1:10:end,1:10:end)',u(1:10:end,1:10:end),v(1:10:end,1:10:end),0);
hold off;

flowmean = invoke(h,'get_flow_mean');
flowvar = invoke(h,'get_flow_var');

disp('flowmean');
disp(flowmean);
disp('flowvar');
disp(flowvar);

disp('CLOSE OBJECT');
invoke(h,'destroy_normal_flow');
delete(h);

%%
%% Copyright (c) 2002, Alexandre Bernardino - ISR/IST.  All rights reserved.
%% alex@isr.ist.utl.pt
%%

