// C++ Sample file to test ActiveX Server Object CornerDetector.CoCornerDetector

#include <stdio.h>
#include <conio.h>
//REQUIRED FOR COM EXTENDED FUNCTIONS
#define _WIN32_WINNT 0x0400
//LOAD TYPE INFORMATION DATA
#import "NormalFlow.tlb" no_namespace named_guids

//THIS PROJECT REQUIRES MICROSOFT VISION SDK. 
//PLEASE MAKE SURE TO HAVE IT INSTALLED ON YOUR MACHINE
#include "viscore.h"

main()
{
	//INITIALIZE COM
	HRESULT hr = CoInitializeEx(NULL, COINIT_MULTITHREADED);
	if(FAILED(hr))
	{
		printf("Unable to Initialize COM\n");
		return -1;
	}

	//CREATE ACTIVEX OBJECT
	INormalFlow *ptr;
	hr = CoCreateInstance(	CLSID_CoNormalFlow, NULL, CLSCTX_INPROC_SERVER, 
							IID_INormalFlow, (void**)&ptr);	
	if(FAILED(hr))
	{
		printf("Unable to Create Object\n");
		return -1;
	}

	CVisGrayByteImage gray1, gray2;
	printf("Reading image sequence ...\n");
	if (!gray1.FReadFile("image1.bmp"))
	{
		printf("Could not open image\n");
		return -1;
	}
	if (!gray2.FReadFile("image1.bmp"))
	{
		printf("Could not open image\n");
		return -1;
	}

	//CONFIGURE OBJECT COMPATIBLE WITH IMAGE AND CORNER ARRAY SIZES
	long image_lines = gray1.Height();
	long image_columns = gray1.Width();	
	hr = ptr->put_lines(image_lines);
	hr = ptr->put_columns(image_columns);
	hr = ptr->put_computation_mode(CComBSTR("all"));
	hr = ptr->put_timederivative_mode(CComBSTR("recursive"));
	hr = ptr->put_minvalidgrad(3.0f);
	hr = ptr->put_minvalidtimederiv(1.0f);
	hr = ptr->put_minvalidflowmag(0.01f);


	//hr = ptr->put_max_corners(max_corners);

	//ALLOCATE THE CORNER DETECTOR
	hr = ptr->create_normal_flow();
	if(FAILED(hr))
	{
		printf("Unable to Allocate Internal Object Resources\n");
		return -1;
	}

	//ASK FOR THE DIRECT ACCESS INTERFACE
	_INormalFlow *ptr_direct;
	hr = ptr->QueryInterface(IID__INormalFlow, (void**)&ptr_direct);
	if(FAILED(hr))
	{
		printf("Unable to access direct interface\n");
		return -1;
	}


	//PROCESS DATA
	//long corners_detected = max_corners;
	hr = ptr_direct->_process_data(gray1.MemBlock().PbData());
	if(FAILED(hr))
	{
		printf("Processing data function failed\n");
		return -1;
	}
	hr = ptr_direct->_process_data(gray2.MemBlock().PbData());
	if(FAILED(hr))
	{
		printf("Processing data function failed\n");
		return -1;
	}

	//SHOWING IMAGES
	float *flowmag;
	float *horzflow;
	float *vertflow;
	short *horzderiv;
	short *vertderiv;
	short *timederiv;

	//direct access to internal pointer : do not write on this memory
	hr = ptr_direct->_get_flowmagnitude(&flowmag); 											 
	if(FAILED(hr))
	{
		printf("Unable to access image\n");
		return -1;
	}
	hr = ptr_direct->_get_horzflow(&horzflow); 											 
	if(FAILED(hr))
	{
		printf("Unable to access image\n");
		return -1;
	}
	hr = ptr_direct->_get_vertflow(&vertflow); 											 
	if(FAILED(hr))
	{
		printf("Unable to access image\n");
		return -1;
	}
	hr = ptr_direct->_get_horzderiv(&horzderiv); 											 
	if(FAILED(hr))
	{
		printf("Unable to access image\n");
		return -1;
	}
	hr = ptr_direct->_get_vertderiv(&vertderiv); 											 
	if(FAILED(hr))
	{
		printf("Unable to access image\n");
		return -1;
	}
	hr = ptr_direct->_get_timederiv(&timederiv); 											 
	if(FAILED(hr))
	{
		printf("Unable to access image\n");
		return -1;
	}
	

	
	//NORMALIZE BRIGHTNESS FOR DISPLAY
	bool ok;
	CVisFloatImage flowmagimage(image_columns, image_lines, 1, evisimoptDefault, (byte*)flowmag);
	CVisShortImage horzderivimage(image_columns, image_lines, 1, evisimoptDefault, (byte*)horzderiv);
	CVisByteImage flowmagnorm, horzderivnorm;
	flowmagnorm.Allocate(flowmagimage.Rect());
	horzderivnorm.Allocate(horzderivimage.Rect());
	ok = flowmagimage.CopyPixelsTo(flowmagnorm, evisnormalizeRelative);
	if(!ok)
	{
		printf("Cannot convert image\n");
		return -1;
	}
	ok = horzderivimage.CopyPixelsTo(horzderivnorm, evisnormalizeRelative);
	if(!ok)
	{
		printf("Cannot convert image\n");
		return -1;
	}

	//DISPLAY
	printf("Displaying images\n");
	CVisImageWnd wflownorm(flowmagnorm);
	CVisImageWnd whorzderivnorm(horzderivnorm);
	printf("Press any key to continue...\n");
	_getch();


	//CLOSE EVERYTHING
	ptr->destroy_normal_flow();
	ptr_direct->Release();
	ptr->Release();

	CoUninitialize();
	
	printf("Done\n");
	return 0;
}